package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/model/response"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type CabinetApi struct {
}

// CreateCabinet
// @Tags      手机柜管理
// @Summary   创建柜子
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Cabinet true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet [post]
func (ca *CabinetApi) CreateCabinet(c *gin.Context) {
	var cabinet model.Cabinet
	if err := bind.Bind(c, &cabinet, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(cabinet, utils.CabinetVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	cabinet.CreateBy = global.JWT.GetUserID(c)
	err = cabinetService.CreateCabinet(cabinet)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateCabinet
// @Tags      手机柜管理
// @Summary   更新柜子
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Cabinet true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet [put]
func (ca *CabinetApi) UpdateCabinet(c *gin.Context) {
	var cabinet model.Cabinet
	if err := bind.Bind(c, &cabinet, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(cabinet.Model, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	cabinet.UpdateBy = global.JWT.GetUserID(c)
	err = cabinetService.UpdateCabinet(cabinet)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteCabinet
// @Tags      手机柜管理
// @Summary   删除柜子
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet [delete]
func (ca *CabinetApi) DeleteCabinet(c *gin.Context) {
	var ID common.IdReq
	if err := bind.Bind(c, &ID, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ID, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = cabinetService.DeleteCabinet(ID.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetCabinet
// @Tags      手机柜管理
// @Summary   获取单个柜子
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet [get]
func (ca *CabinetApi) GetCabinet(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	res, err := cabinetService.GetCabinet(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.CabinetResponse{Cabinet: res})
	return
}

// GetCabinetList
// @Tags      手机柜管理
// @Summary   获取柜子列表
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.CabinetSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet/list [get]
func (ca *CabinetApi) GetCabinetList(c *gin.Context) {
	var pageInfo request.CabinetSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	// 数据权限
	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := cabinetService.GetCabinetList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}

// ChangeCabinetStatus
// @Tags      手机柜管理
// @Summary   更新柜子状态
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     request.ChangeCabinetStatusRequest true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinet/status [put]
func (ca *CabinetApi) ChangeCabinetStatus(c *gin.Context) {
	var req request.ChangeCabinetStatusRequest
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = cabinetService.ChangeCabinetStatus(req.ID, req.Status)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}
