package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type CabinetTimeApi struct {
}

// CreateCabinetTime
// @Tags      时间管理
// @Summary   创建
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.CabinetTime true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinetTime [post]
func (api *CabinetTimeApi) CreateCabinetTime(c *gin.Context) {
	var ct model.CabinetTime
	if err := bind.Bind(c, &ct, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ct, utils.CabinetTimeVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	ct.CreateBy = global.JWT.GetUserID(c)
	err = cabinetTimeService.CreateCabinetTime(ct)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteCabinetTime
// @Tags      时间管理
// @Summary   删除
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinetTime [delete]
func (api *CabinetTimeApi) DeleteCabinetTime(c *gin.Context) {
	var ID common.IdReq
	if err := bind.Bind(c, &ID, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ID, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = cabinetTimeService.DeleteCabinetTime(ID.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetCabinetTimeList
// @Tags      时间管理
// @Summary   列表
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     request.CabinetTimeSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinetTime [get]
func (api *CabinetTimeApi) GetCabinetTimeList(c *gin.Context) {
	var pageInfo request.CabinetTimeSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, total, err := cabinetTimeService.GetCabinetTimeList(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}

// ChangeUseStatus
// @Tags      时间管理
// @Summary   启用或者禁用
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     request.ChangeUseStatusRequest true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/cabinetTime [put]
func (api *CabinetTimeApi) ChangeUseStatus(c *gin.Context) {
	var req request.ChangeUseStatusRequest
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(req, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = cabinetTimeService.ChangeUseStatus(req.ID, req.Status)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}
