package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type GridOpenLogApi struct {
}

// CreateGridOpenLog
// @Tags      日志管理
// @Summary   创建开关门日志(测试使用)
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    model.GridOpenLog true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/open [post]
func (api *GridOpenLogApi) CreateGridOpenLog(c *gin.Context) {
	var log model.GridOpenLog
	if err := bind.Bind(c, &log, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := gridOpenLogService.CreateGridOpenLog(log)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteGridOpenLogByIds
// @Tags      日志管理
// @Summary   批量删除开关门日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/open [delete]
func (api *GridOpenLogApi) DeleteGridOpenLogByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = gridOpenLogService.DeleteGridOpenLogByIds(IDS)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetGridOpenLogList
// @Tags      日志管理
// @Summary   分页获取开关门日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridOpenLogSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/open/list [get]
func (api *GridOpenLogApi) GetGridOpenLogList(c *gin.Context) {
	var pageInfo request.GridOpenLogSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	// 数据权限
	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := gridOpenLogService.GetGridOpenLogList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}

// GetGridOpenLogStatistics
// @Tags      日志管理
// @Summary   开关门次数统计
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridOpenLogStatistics true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/open/statistics [get]
func (api *GridOpenLogApi) GetGridOpenLogStatistics(c *gin.Context) {
	var pageInfo request.GridOpenLogStatistics
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, err := gridOpenLogService.GetGridOpenLogStatistics(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List: list,
	})
	return
}
