package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/model/response"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type StaffApi struct {
}

// CreateStaff
// @Tags      人员管理
// @Summary   创建人员
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Staff true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/staff [post]
func (s *StaffApi) CreateStaff(c *gin.Context) {
	var staff model.Staff
	if err := bind.Bind(c, &staff, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(staff, utils.StaffVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	staff.CreateBy = global.JWT.GetUserID(c)
	err = staffService.CreateStaff(staff)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateStaff
// @Tags      人员管理
// @Summary   更新人员
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Staff true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/staff [put]
func (s *StaffApi) UpdateStaff(c *gin.Context) {
	var staff model.Staff
	if err := bind.Bind(c, &staff, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(staff.Model, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	staff.UpdateBy = global.JWT.GetUserID(c)
	err = staffService.UpdateStaff(staff)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteStaff
// @Tags      人员管理
// @Summary   删除人员
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/staff [delete]
func (s *StaffApi) DeleteStaff(c *gin.Context) {
	var ID common.IdReq
	if err := bind.Bind(c, &ID, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ID, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = staffService.DeleteStaff(ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetStaff
// @Tags      人员管理
// @Summary   获取人员
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/staff [get]
func (s *StaffApi) GetStaff(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	res, err := staffService.GetStaff(idReq)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.StaffResponse{Staff: res})
	return
}

// GetStaffList
// @Tags      人员管理
// @Summary   分页获取人员
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     request.StaffSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/staff/list [get]
func (s *StaffApi) GetStaffList(c *gin.Context) {
	var pageInfo request.StaffSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	// 数据权限
	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := staffService.GetStaffList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}
