package dataProto

import (
	"bytes"
	"encoding/binary"
	"fmt"
	"reflect"
)

// structToBytes 结构转字节数组 适用于通讯协议编码
// note: 仅适用此项目.结构体最后一个属性为CRCCode。
func structToBytes(i interface{}) ([]byte, error) {
	var v reflect.Value
	t := reflect.TypeOf(i)
	if t.Kind() == reflect.Ptr {
		v = reflect.ValueOf(i).Elem()
	} else {
		v = reflect.ValueOf(i)
	}
	buf := bytes.NewBuffer([]byte{})
	for i := 0; i < v.NumField()-1; i++ {
		if v.Field(i).Type().Name() == "CRCCode" {
			continue
		}
		err := binary.Write(buf, binary.BigEndian, v.Field(i).Interface())
		if err != nil {
			return nil, err
		}
	}

	crcValue := uint16(v.FieldByName("CRCCode").Uint())
	var crc uint16
	if crcValue == 0 {
		//计算CRC
		crc = calCRCCode(buf.Bytes())
	} else {
		crc = crcValue
	}

	err := binary.Write(buf, binary.BigEndian, crc)
	if err != nil {
		return nil, err
	}
	return buf.Bytes(), nil
}

// bytesToStruct 字节数组转结构,适用于通讯协议解码。
// note: 仅适用此项目。结构体数据全部为基本数据类型。如果结构中有复合数据类型,此函数不适用。
func bytesToStruct(buf []byte, i interface{}) error {
	//crc检测
	pass, err := checkCRCCode(buf)
	if err != nil {
		return err
	}

	if !pass {
		return fmt.Errorf("crc code check err")
	}

	t := reflect.TypeOf(i)
	if t.Kind() != reflect.Ptr {
		return fmt.Errorf("struct type must be a pointer")
	}
	v := reflect.ValueOf(i).Elem()

	start := 0
	for i := 0; i < v.NumField(); i++ {
		fieldBytes := v.Field(i).Type().Bits() / 8
		fieldDataBuf := bytes.NewBuffer(buf[start : start+fieldBytes])
		start += fieldBytes
		switch v.Field(i).Kind() {
		case reflect.Uint8:
			var fieldData uint8
			err := binary.Read(fieldDataBuf, binary.BigEndian, &fieldData)
			if err != nil {
				fmt.Println(err)
				return err
			}
			v.Field(i).SetUint(uint64(fieldData))
		case reflect.Uint16:
			var fieldData uint16
			err := binary.Read(fieldDataBuf, binary.BigEndian, &fieldData)
			if err != nil {
				fmt.Println(err)
				return err
			}
			v.Field(i).SetUint(uint64(fieldData))
		case reflect.Uint32:
			var fieldData uint32
			err := binary.Read(fieldDataBuf, binary.BigEndian, &fieldData)
			if err != nil {
				fmt.Println(err)
				return err
			}
			v.Field(i).SetUint(uint64(fieldData))
		case reflect.Uint64:
			var fieldData uint64
			err := binary.Read(fieldDataBuf, binary.BigEndian, &fieldData)
			if err != nil {
				fmt.Println(err)
				return err
			}
			v.Field(i).SetUint(fieldData)
		default:
		}
	}
	return nil
}

func calCRCCode(srcBuf []byte) uint16 {
	var xCRC uint16 = 0xffff
	for i := 0; i < len(srcBuf); i++ {
		xCRC = calCRCModeBus(uint16(srcBuf[i]), xCRC)
	}
	crc_Hi := xCRC % 256
	crc_Lo := xCRC / 256
	xCRC = crc_Hi
	xCRC = (xCRC << 8) | crc_Lo
	return xCRC
}

func calCRCModeBus(td uint16, dCRC uint16) uint16 {
	var a2 uint16 = 0
	dCRC = dCRC ^ td
	for i := 0; i < 8; i++ {
		a2 = dCRC & 1
		dCRC = dCRC >> 1
		dCRC = dCRC & 0x7FFF

		if a2 == 1 {
			dCRC ^= 0xa001
		}

		dCRC = dCRC & 0xffff
	}
	return dCRC
}

func checkCRCCode(pkgBuf []byte) (bool, error) {
	var crcCode uint16
	var buf = bytes.NewBuffer(pkgBuf[len(pkgBuf)-2:])
	err := binary.Read(buf, binary.BigEndian, &crcCode)
	if err != nil {
		return false, err
	}

	if crcCode == 0x1DC0 {
		return true, nil // 固定值  直接返回
	}

	code := calCRCCode(pkgBuf[:len(pkgBuf)-2])
	return code == crcCode, nil
}

// CabinetConfigProtoEncode 配置柜子编号编码
func CabinetConfigProtoEncode(ccp *CabinetConfigProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if ccp == nil {
		return nil, fmt.Errorf("cabinetConfigProto is nil")
	}

	// 检测命令
	if ccp.Cmd != CmdCabinetConfig {
		return nil, fmt.Errorf("cabinetConfigProto cmd expect %X,but give %X", CmdCabinetConfig, ccp.Cmd)
	}

	return structToBytes(ccp)
}

// CabinetConfigProtoDecode 配置柜子编号解码
func CabinetConfigProtoDecode(protoBytes []byte) (*CabinetConfigProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthCabinetConfig {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthCabinetConfig != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	ccp := &CabinetConfigProto{}
	err := bytesToStruct(protoBytes, ccp)
	if err != nil {
		return nil, err
	}

	return ccp, nil
}

// GridControlProtoEncode 控制格子允许开 禁止开编码
func GridControlProtoEncode(gcp *GridControlProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if gcp == nil {
		return nil, fmt.Errorf("doorControlProto is nil")
	}

	return structToBytes(gcp)
}

func GridControlProtoDecode(protoBytes []byte) (*GridControlProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthGridControl {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthGridControl != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	gcp := &GridControlProto{}
	err := bytesToStruct(protoBytes, gcp)
	if err != nil {
		return nil, err
	}

	return gcp, err
}

// DoorControlProtoEncode 控制柜子门允许开 禁止开 编码
func DoorControlProtoEncode(dcp *DoorControlProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if dcp == nil {
		return nil, fmt.Errorf("doorControlProto is nil")
	}

	return structToBytes(dcp)
}

// DoorControlProtoDecode 控制柜子门允许开 禁止开 解码
func DoorControlProtoDecode(protoBytes []byte) (*DoorControlProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthDoorControl {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthDoorControl != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	dcp := &DoorControlProto{}
	err := bytesToStruct(protoBytes, dcp)
	if err != nil {
		return nil, err
	}

	return dcp, err
}

// OpenGridProtoEncode 打开箱门编码
func OpenGridProtoEncode(ogp *OpenGridProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if ogp == nil {
		return nil, fmt.Errorf("openGridProto is nil")
	}

	// 检测开始符号
	if ogp.StartSymbol != StartSymbolOpenGrid {
		return nil, fmt.Errorf("openGridProto start symbol expect %X,but give %X", StartSymbolOpenGrid, ogp.StartSymbol)
	}

	// 检测命令
	if ogp.Cmd != CmdOpenGrid {
		return nil, fmt.Errorf("cabinetConfigProto cmd expect %X,but give %X", CmdOpenGrid, ogp.Cmd)
	}

	return structToBytes(ogp)
}

// OpenGridProtoDecode 打开箱门解码
func OpenGridProtoDecode(protoBytes []byte) (*OpenGridProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthOpenGrid {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthOpenGrid != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	ogp := &OpenGridProto{}
	err := bytesToStruct(protoBytes, ogp)
	if err != nil {
		return nil, err
	}

	return ogp, err
}

// GridLockStatusProtoEncode 箱门锁状态编码
func GridLockStatusProtoEncode(gsp *GridLockStatusProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if gsp == nil {
		return nil, fmt.Errorf("gridLockStatusProto is nil")
	}

	// 检测开始符号
	if gsp.StartSymbol != StartSymbolGridLockStatusOrGridStatus {
		return nil, fmt.Errorf("gridLockStatusProto start symbol expect %X,but give %X", StartSymbolOpenGrid, gsp.StartSymbol)
	}

	// 检测命令
	if gsp.Cmd != CmdGridLockStatus {
		return nil, fmt.Errorf("gridLockStatusProto cmd expect %X,but give %X", CmdOpenGrid, gsp.Cmd)
	}

	return structToBytes(gsp)
}

// GridLockStatusProtoDecode 箱门锁状态解码
func GridLockStatusProtoDecode(protoBytes []byte) (*GridLockStatusProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthGridLockStatus {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthGridLockStatus != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	gsp := &GridLockStatusProto{}
	err := bytesToStruct(protoBytes, gsp)
	if err != nil {
		return nil, err
	}

	return gsp, err
}

// GetCabinetReqEncode 获取柜子信息请求编码
func GetCabinetReqEncode(gcr *GetCabinetReq) ([]byte, error) {
	// 检测编码数据 不许为空
	if gcr == nil {
		return nil, fmt.Errorf("getCabinetReq is nil")
	}

	// 检测开始符号
	if gcr.StartSymbol != StartSymbolGetCabinetReq {
		return nil, fmt.Errorf("getCabinetReq start symbol expect %X,but give %X", StartSymbolOpenGrid, gcr.StartSymbol)
	}

	// 检测命令
	if gcr.Cmd != CmdGetCabinetReq {
		return nil, fmt.Errorf("getCabinetReq cmd expect %X,but give %X", CmdOpenGrid, gcr.Cmd)
	}

	return structToBytes(gcr)
}

// GetCabinetReqDecode 获取柜子信息请求解码
func GetCabinetReqDecode(protoBytes []byte) (*GetCabinetReq, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthGetCabinetReq {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthGetCabinetReq != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	gcr := &GetCabinetReq{}
	err := bytesToStruct(protoBytes, gcr)
	if err != nil {
		return nil, err
	}

	return gcr, err
}

// GridStatusProtoEncode 获取柜子信息响应编码
func GridStatusProtoEncode(gcr *GridStatsProto) ([]byte, error) {
	// 检测编码数据 不许为空
	if gcr == nil {
		return nil, fmt.Errorf("getCabinetRes is nil")
	}

	// 检测开始符号
	if gcr.StartSymbol != StartSymbolGridLockStatusOrGridStatus {
		return nil, fmt.Errorf("getCabinetRes start symbol expect %X,but give %X", StartSymbolOpenGrid, gcr.StartSymbol)
	}

	// 检测命令
	if gcr.Cmd != CmdGetCabinetRes {
		return nil, fmt.Errorf("getCabinetRes cmd expect %X,but give %X", CmdOpenGrid, gcr.Cmd)
	}

	return structToBytes(gcr)
}

// GridStatusProtoDecode 获取柜子信息响应解码
func GridStatusProtoDecode(protoBytes []byte) (*GridStatsProto, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthGetCabinetRes {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthGetCabinetRes != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	gcr := &GridStatsProto{}
	err := bytesToStruct(protoBytes, gcr)
	if err != nil {
		return nil, err
	}

	return gcr, err
}

// GetDoorStatusReqEncode 下位机获取柜子门当前状态请求编码
func GetDoorStatusReqEncode(gds *GetDoorStatusReq) ([]byte, error) {
	// 检测编码数据 不许为空
	if gds == nil {
		return nil, fmt.Errorf("getDoorStatusReq is nil")
	}

	// 检测开始符号
	if gds.StartSymbol != StartSymbolGetDoorStatusReq {
		return nil, fmt.Errorf("getDoorStatusReq start symbol expect %X,but give %X", StartSymbolOpenGrid, gds.StartSymbol)
	}

	// 检测命令
	if gds.Cmd != CmdGetDoorStatusReq {
		return nil, fmt.Errorf("getDoorStatusReq cmd expect %X,but give %X", CmdOpenGrid, gds.Cmd)
	}

	return structToBytes(gds)
}

// GetDoorStatusReqDecode 下位机获取柜子门当前状态请求解码
func GetDoorStatusReqDecode(protoBytes []byte) (*GetDoorStatusReq, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return nil, fmt.Errorf("protoBytes is nil or 0")
	}

	if int(protoBytes[0]) != PkgLengthGetDoorStatusReq {
		return nil, fmt.Errorf("pkgLength expect %d,but give %d", PkgLengthCabinetConfig, protoBytes[0])
	}

	if PkgLengthGetDoorStatusReq != len(protoBytes) {
		return nil, fmt.Errorf("protoBytes len expect %d,but actual is %d", protoBytes[0], len(protoBytes))
	}

	gds := &GetDoorStatusReq{}
	err := bytesToStruct(protoBytes, gds)
	if err != nil {
		return nil, err
	}

	return gds, err
}

// GetPkgStartSymbol 获取包的开始标志，用来判断是什么数据包
func GetPkgStartSymbol(protoBytes []byte) (uint8, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return 0, fmt.Errorf("pkg is nil")
	}

	if len(protoBytes) < 2 {
		return 0, fmt.Errorf("pkg is err")
	}

	return protoBytes[1], nil
}

// GetPkgCmd 获取数据包的命令字
func GetPkgCmd(protoBytes []byte) (uint8, error) {
	if protoBytes == nil || len(protoBytes) == 0 {
		return 0, fmt.Errorf("pkg is nil")
	}

	if len(protoBytes) < 3 {
		return 0, fmt.Errorf("pkg is err")
	}

	return protoBytes[2], nil
}
