package my_middleware

import (
	"bytes"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/service"
	"dy-admin/pkg/log"
	"encoding/json"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"io"
	"net/http"
	"net/url"
	"strings"
	"sync"
)

var operationRecordService = service.ServicesGroupApp.OperationRecordService

var respPool sync.Pool

func init() {
	respPool.New = func() interface{} {
		return make([]byte, 128)
	}
}

func OperationRecord() gin.HandlerFunc {
	return func(c *gin.Context) {
		var body []byte
		var userId int
		if c.Request.Method != http.MethodGet {
			var err error
			body, err = io.ReadAll(c.Request.Body)
			if err != nil {
				log.Error("read body from request error:", zap.Error(err))
			} else {
				c.Request.Body = io.NopCloser(bytes.NewBuffer(body))
			}
		} else {
			query := c.Request.URL.RawQuery
			query, _ = url.QueryUnescape(query)
			split := strings.Split(query, "&")
			m := make(map[string]string)
			for _, v := range split {
				kv := strings.Split(v, "=")
				if len(kv) == 2 {
					m[kv[0]] = kv[1]
				}
			}
			body, _ = json.Marshal(&m)
		}
		userId = global.JWT.GetUserID(c)
		record := model.OperationRecord{
			Ip:        c.ClientIP(),
			Method:    c.Request.Method,
			Path:      c.Request.URL.Path,
			Agent:     c.Request.UserAgent(),
			Body:      string(body),
			NickName:  global.JWT.GetUserInfo(c).NickName,
			ControlBy: common.ControlBy{CreateBy: userId},
		}

		// 上传文件时候 中间件日志进行裁断操作
		if strings.Contains(c.GetHeader("Content-Type"), "multipart/form-data") {
			if len(record.Body) > 1024 {
				// 截断
				newBody := respPool.Get().([]byte)
				copy(newBody, record.Body)
				record.Body = string(newBody)
				defer respPool.Put(newBody[:0])
			}
		}

		writer := responseBodyWriter{
			ResponseWriter: c.Writer,
			body:           &bytes.Buffer{},
		}
		c.Writer = writer

		c.Next()

		go func() {
			record.ErrorMessage = c.Errors.ByType(gin.ErrorTypePrivate).String()
			record.Status = c.Writer.Status()
			record.Resp = writer.body.String()

			if strings.Contains(c.Writer.Header().Get("Pragma"), "public") ||
				strings.Contains(c.Writer.Header().Get("Expires"), "0") ||
				strings.Contains(c.Writer.Header().Get("Cache-Control"), "must-revalidate, post-check=0, pre-check=0") ||
				strings.Contains(c.Writer.Header().Get("Content-Type"), "application/force-download") ||
				strings.Contains(c.Writer.Header().Get("Content-Type"), "application/octet-stream") ||
				strings.Contains(c.Writer.Header().Get("Content-Type"), "application/vnd.ms-excel") ||
				strings.Contains(c.Writer.Header().Get("Content-Type"), "application/download") ||
				strings.Contains(c.Writer.Header().Get("Content-Disposition"), "attachment") ||
				strings.Contains(c.Writer.Header().Get("Content-Transfer-Encoding"), "binary") {
				if len(record.Resp) > 128 {
					// 截断
					newBody := respPool.Get().([]byte)
					copy(newBody, record.Resp)
					record.Resp = string(newBody)
					defer respPool.Put(newBody[:0])
				}
			}

			if err := operationRecordService.CreateSysOperationRecord(record); err != nil {
				log.Error("create operation record error:", zap.Error(err))
			}
		}()
	}
}

type responseBodyWriter struct {
	gin.ResponseWriter
	body *bytes.Buffer
}

func (r responseBodyWriter) Write(b []byte) (int, error) {
	r.body.Write(b)
	return r.ResponseWriter.Write(b)
}
