package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/model/request"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type ApiApi struct {
}

// CreateApi
// @Tags      系统api管理
// @Summary   创建api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Api                  true  "api路径, api中文描述, api组, 方法"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/api [post]
func (s *ApiApi) CreateApi(c *gin.Context) {
	var api model.Api
	if err := bind.Bind(c, &api, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(api, utils.ApiVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = apiService.CreateApi(api)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteApi
// @Tags      系统api管理
// @Summary   删除api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.IdReq                 true  "ID"
// @Success   200   {object}  rescode.ErrCode "删除api"
// @Router    /sys/v1/api [delete]
func (s *ApiApi) DeleteApi(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = apiService.DeleteApi(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetApiList
// @Tags      系统api管理
// @Summary   分页获取api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.SearchApiParams                              true  "分页获取API列表"
// @Success   200   {object}  rescode.ErrCode  "分页获取API列表,返回包括列表,总数,页码,每页数量"
// @Router    /sys/v1/api/list [get]
func (s *ApiApi) GetApiList(c *gin.Context) {
	var pageInfo request.SearchApiParams
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := apiService.GetAPIInfoList(pageInfo.Api, pageInfo.PageInfo, pageInfo.OrderKey, pageInfo.Desc)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
}

// GetApiById
// @Tags      系统api管理
// @Summary   根据id获取api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.IdReq                                  true  "根据id获取api"
// @Success   200   {object}  rescode.ErrCode  "根据id获取api,返回包括api详情"
// @Router    /sys/v1/api/one [get]
func (s *ApiApi) GetApiById(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	api, err := apiService.GetApiById(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.APIResponse{Api: api})
}

// UpdateApi
// @Tags      系统api管理
// @Summary   修改基础api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      model.Api                 true  "api路径, api中文描述, api组, 方法"
// @Success   200   {object}  rescode.ErrCode  "修改基础api"
// @Router    /sys/v1/api  [put]
func (s *ApiApi) UpdateApi(c *gin.Context) {
	var api model.Api
	if err := bind.Bind(c, &api, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(api, utils.ApiVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = apiService.UpdateApi(api)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteApisByIds
// @Tags      系统api管理
// @Summary   删除选中Api
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.IdsReq                 true  "ID"
// @Success   200   {object}  rescode.ErrCode  "删除选中Api"
// @Router    /sys/v1/api/ids  [delete]
func (s *ApiApi) DeleteApisByIds(c *gin.Context) {
	var ids common.IdsReq
	if err := bind.Bind(c, &ids, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := apiService.DeleteApisByIds(ids)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}
