package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type DeptApi struct {
}

// CreateDept
// @Tags      系统部门管理
// @Summary   创建部门
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Dept  true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/dept [post]
func (s *DeptApi) CreateDept(c *gin.Context) {
	var dept model.Dept
	if err := bind.Bind(c, &dept, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(dept, utils.DeptVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	dept.CreateBy = global.JWT.GetUserID(c)
	err = deptService.CreateDept(dept)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateDept
// @Tags      系统部门管理
// @Summary   更新部门
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Dept true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/dept [put]
func (s *DeptApi) UpdateDept(c *gin.Context) {
	var dept model.Dept
	if err := bind.Bind(c, &dept, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(dept, utils.DeptVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	dept.UpdateBy = global.JWT.GetUserID(c)
	err = deptService.UpdateDept(dept)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteDept
// @Tags      系统部门管理
// @Summary   删除部门
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/dept [delete]
func (s *DeptApi) DeleteDept(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = deptService.DeleteDept(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetDeptTree
// @Tags      系统部门管理
// @Summary   获取部门树
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/dept/tree [get]
func (s *DeptApi) GetDeptTree(c *gin.Context) {
	list, err := deptService.GetDepts()
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.DeptListResponse{Depts: list})
}

// GetDept
// @Tags      系统部门管理
// @Summary   获取部门
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/dept/one [get]
func (s *DeptApi) GetDept(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	//err := utils.Verify(idReq, utils.IdVerify)
	//if err != nil {
	//	err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
	//	rescode.WriteErrResponse(c, err)
	//	return
	//}
	dept, err := deptService.GetDeptById(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.DeptResponse{Dept: dept})
}
