package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model/request"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type LoginRecordApi struct{}

// DeleteLoginRecordByIds
// @Tags      登录日志管理
// @Summary   批量删除登录日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/loginRecord/ids [delete]
func (lr *LoginRecordApi) DeleteLoginRecordByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = loginRecordService.DeleteLoginRecordByIDs(IDS)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// FindLoginRecord
// @Tags      登录日志管理
// @Summary   获取单条日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/loginRecord [get]
func (lr *LoginRecordApi) FindLoginRecord(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	reSysOperationRecord, err := loginRecordService.GetLoginRecord(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.LoginRecordResponse{LoginRecord: reSysOperationRecord})
	return
}

// GetLoginRecordList
// @Tags      登录日志管理
// @Summary   分页获取日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.LoginRecordSearch  true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/loginRecord/list [get]
func (lr *LoginRecordApi) GetLoginRecordList(c *gin.Context) {
	var pageInfo request.LoginRecordSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, total, err := loginRecordService.GetLoginRecordList(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}
