package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/model/request"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type RoleApi struct {
}

// CreateRole
// @Tags      系统角色管理
// @Summary   创建角色
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Role true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role [post]
func (s *RoleApi) CreateRole(c *gin.Context) {
	var role model.Role
	if err := bind.Bind(c, &role, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(role, utils.RoleVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	role.CreateBy = global.JWT.GetUserID(c)
	err = roleService.CreateRole(role)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateRole
// @Tags      系统角色管理
// @Summary   更新角色
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.Role true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role [put]
func (s *RoleApi) UpdateRole(c *gin.Context) {
	var role model.Role
	if err := bind.Bind(c, &role, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(role, utils.RoleIdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	role.UpdateBy = global.JWT.GetUserID(c)
	err = roleService.UpdateRole(role)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteRole
// @Tags      系统角色管理
// @Summary   删除角色
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.GetRoleID true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role [delete]
func (s *RoleApi) DeleteRole(c *gin.Context) {
	var idReq common.GetRoleID
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.RoleIdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = roleService.DeleteRole(idReq.RoleID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetRoleByID
// @Tags      系统角色管理
// @Summary   获取角色
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.GetRoleID true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role/one [get]
func (s *RoleApi) GetRoleByID(c *gin.Context) {
	var idReq common.GetRoleID
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.RoleIdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	role, err := roleService.GetRoleById(idReq.RoleID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.RoleResponse{Role: role})
}

// GetRoles
// @Tags      系统角色管理
// @Summary   分页获取角色
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.SearchRoleParams                             true  "分页获取API列表"
// @Success   200   {object}  rescode.ErrCode  "分页获取API列表,返回包括列表,总数,页码,每页数量"
// @Router    /sys/v1/role/list [get]
func (s *RoleApi) GetRoles(c *gin.Context) {
	var pageInfo request.SearchRoleParams
	if err := bind.Bind(c, &pageInfo, binding.JSON, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := roleService.GetRoleList(pageInfo.Role, pageInfo.PageInfo, pageInfo.OrderKey, pageInfo.Desc)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
}

// GetRoleDeptIDs
// @Tags      系统角色管理
// @Summary   角色当前部门ids
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.GetRoleID                             true  "分页获取API列表"
// @Success   200   {object}  rescode.ErrCode  "分页获取API列表,返回包括列表,总数,页码,每页数量"
// @Router    /sys/v1/role/deptIds [get]
func (s *RoleApi) GetRoleDeptIDs(c *gin.Context) {
	var idReq common.GetRoleID
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	deptIds, err := roleService.GetRoleDeptIDs(idReq.RoleID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.RoleDeptIDsResponse{DeptIDs: deptIds})
}

// GetRoleMenuIDs
// @Tags      系统角色管理
// @Summary   角色当前菜单ids
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.GetRoleID true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role/menuIds [get]
func (s *RoleApi) GetRoleMenuIDs(c *gin.Context) {
	var idReq common.GetRoleID
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	menuIds, err := roleService.GetRoleMenuIDs(idReq.RoleID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.RoleMenuIDsResponse{MenuIDs: menuIds})
}

// UpdateDataScope
// @Tags      系统角色管理
// @Summary   更新角色数据权限
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.RoleDataScopeReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/role/dataScope [put]
func (s *RoleApi) UpdateDataScope(c *gin.Context) {
	var req request.RoleDataScopeReq
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.RoleDataScopeVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = roleService.UpdateDataScope(req.RoleId, req.DataScope, req.DeptIds)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}
