package service

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/pkg/rescode"
	"github.com/pkg/errors"

	"fmt"
	"gorm.io/gorm"
)

type ApiService struct {
}

func (apiService *ApiService) CreateApi(api model.Api) (err error) {
	err = global.DB.Where("path = ? AND action = ?", api.Path, api.Action).First(&model.Api{}).Error
	if !errors.Is(err, gorm.ErrRecordNotFound) {
		msg := fmt.Sprintf("params=>path:%s,action:%s", api.Path, api.Action)
		return rescode.RegisterErrorWithMessage(errors.Wrap(err, msg), msg, code.ErrCodeMap[code.ErrApiAlreadyExistCode])
	}
	err = global.DB.Create(&api).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	return
}

func (apiService *ApiService) DeleteApi(id int) (err error) {
	var entity model.Api
	err = global.DB.Where("id = ?", id).First(&entity).Error
	if errors.Is(err, gorm.ErrRecordNotFound) {
		msg := fmt.Sprintf("params=>id:%d", id)
		return rescode.RegisterErrorWithMessage(errors.Wrap(err, msg), msg, code.ErrCodeMap[code.ErrApiNotFoundCode])
	}

	// api是否有被菜单使用
	var count int64
	err = global.DB.Table("sys_menu_apis").Where("api_id = ?", id).Count(&count).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	if count != 0 {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDeleteMenuUseCode])
	}

	err = global.DB.Delete(&entity).Error
	if err != nil {
		msg := fmt.Sprintf("params=>id:%d", id)
		return rescode.RegisterErrorWithMessage(err, msg, code.ErrCodeMap[code.ErrDatabaseCode])
	}

	success, err := CasbinServiceInstance.ClearCasbin(1, entity.Path, entity.Action)
	if !success {
		msg := fmt.Sprintf("params=>path:%s,action:%s", entity.Path, entity.Action)
		return rescode.RegisterErrorWithMessage(err, msg, code.ErrCodeMap[code.ErrCasbinInternalCode])
	}
	e := CasbinServiceInstance.Casbin()
	err = e.InvalidateCache()
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrCasbinInternalCode])
	}
	return
}

func (apiService *ApiService) GetAPIInfoList(api model.Api, info common.PageInfo, order string, desc bool) (list interface{}, total int64, err error) {
	var limit, offset int
	if info.Page == 0 || info.PageSize == 0 {
		limit = -1
		offset = -1
	} else {
		limit = info.PageSize
		offset = info.PageSize * (info.Page - 1)
	}

	db := global.DB.Model(&model.Api{})
	var apiList []model.Api

	if api.Path != "" {
		db = db.Where("path LIKE ?", "%"+api.Path+"%")
	}

	if api.Description != "" {
		db = db.Where("description LIKE ?", "%"+api.Description+"%")
	}

	if api.Action != "" {
		db = db.Where("action = ?", api.Action)
	}

	if api.ApiGroup != "" {
		db = db.Where("api_group = ?", api.ApiGroup)
	}

	if api.Type != "" {
		db = db.Where("type = ?", api.Type)
	}

	err = db.Count(&total).Error
	if err != nil {
		return apiList, total, rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}

	db = db.Limit(limit).Offset(offset)
	if order != "" {
		var OrderStr string
		allowOrderField := api.AllowOrderField()
		if _, ok := allowOrderField[order]; !ok {

			msg := fmt.Sprintf("params=>order:%d", order)
			return apiList, total,
				rescode.RegisterErrorWithMessage(errors.New("非法排序字段"), msg, code.ErrCodeMap[code.ErrIllegalOrderFieldCode])

		}
		if desc {
			OrderStr = order + " desc"
		} else {
			OrderStr = order
		}
		err = db.Order(OrderStr).Find(&apiList).Error
	} else {
		err = db.Order("id").Find(&apiList).Error
	}
	return apiList, total, nil
}

func (apiService *ApiService) GetApiById(id int) (api model.Api, err error) {
	err = global.DB.Where("id = ?", id).First(&api).Error
	if err != nil {
		return api, rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	return
}

func (apiService *ApiService) UpdateApi(api model.Api) (err error) {
	var oldA model.Api
	err = global.DB.Where("id = ?", api.ID).First(&oldA).Error
	if oldA.Path != api.Path || oldA.Action != api.Action {
		err = global.DB.Where("path = ? AND action = ?", api.Path, api.Action).First(&model.Api{}).Error
		if err != nil {
			if !errors.Is(err, gorm.ErrRecordNotFound) {
				msg := fmt.Sprintf("params=>%s:%s", api.Path, api.Action)
				return rescode.RegisterErrorWithMessage(err, msg, code.ErrCodeMap[code.ErrApiAlreadyExistCode])
			}
		} else {
			return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
		}

	}

	err = CasbinServiceInstance.UpdateCasbinApi(oldA.Path, api.Path, oldA.Action, api.Action)
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrCasbinInternalCode])
	}

	err = global.DB.Updates(&api).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	return
}

func (apiService *ApiService) DeleteApisByIds(ids common.IdsReq) (err error) {
	// api是否有被菜单使用
	var count int64
	err = global.DB.Table("sys_menu_apis").Where("api_id in ?", ids.IDs).Count(&count).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	if count != 0 {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDeleteMenuUseCode])
	}

	var apis []model.Api
	err = global.DB.Find(&apis, "id in ?", ids.IDs).Delete(&apis).Error
	if err != nil {
		return err
	}

	for _, sysApi := range apis {
		_, _ = CasbinServiceInstance.ClearCasbin(1, sysApi.Path, sysApi.Action)
	}
	err = CasbinServiceInstance.Casbin().InvalidateCache()
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrCasbinInternalCode])
	}

	return
}
